(function (Drupal) {
  'use strict';

  Drupal.behaviors.adaptiveImageWrappers = {
    images: [],
    attach: adaptiveImageWrappersizes,
  };

  function adaptiveImageWrappersizes(context) {
    const adaptiveImageWrappers = context.querySelectorAll('.adaptive-image');
    // Collect the images.
    adaptiveImageWrappers.forEach((adaptiveImageWrapper) => {
      this.images.push(adaptiveImageWrapper);
    });
    // Create a resize observer to update the image sizes.
    const [body] = once('adaptive-image-wrapper', 'body', context);
    if (body) {
      new ResizeObserver(Drupal.debounce(() => {
        this.images.forEach((adaptiveImageWrapper) => {
          imageSizes(adaptiveImageWrapper);
        });
      }, 250)).observe(body);
    }
  }

  // Function to set image sizes
  function imageSizes(adaptiveImageWrapper) {
    const image = adaptiveImageWrapper.querySelector('img');

    if (image) {
      const imageWidth = Math.floor(adaptiveImageWrapper.getBoundingClientRect().width);
      const allImages = [image, ...adaptiveImageWrapper.querySelectorAll('img, source')];

      allImages.forEach((el) => {
        el.setAttribute('sizes', `${imageWidth}px`);
        el.setAttribute('data-sizes', `${imageWidth}px`);
      });
    }
  }

})(Drupal);
